/* -*- Mode: C; c-file-style: "gnu" -*-
   object.c -- native methods for java/lang/Object.
   Created: Chris Toshok <toshok@hungry.com>, 28-Jul-1997
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#include "jni.h"
#include "native-threads.h"
#include "interp.h"
#include "objects.h"
#include "ClazzFile.h"
#include "exceptions.h"

JNIEXPORT jobject JNICALL
Java_java_lang_Object_getClass(JNIEnv *env,
			       jobject obj)
{
  jclass clazz = (*env)->GetObjectClass(env, obj);

  return new_classobject(env, clazz);
}

JNIEXPORT jint JNICALL
Java_java_lang_Object_hashCode(JNIEnv *env,
			       jobject obj)
{
  japhar_object* jobj;

  obj_to_object(obj, jobj);

  return jobj->hash_code;
}

JNIEXPORT jobject JNICALL
Java_java_lang_Object_clone(JNIEnv *env,
			    jobject obj)
{

  return (jobject)clone_object(env, (japhar_obj)obj);
}

JNIEXPORT void JNICALL
Java_java_lang_Object_notify(JNIEnv *env,
			     jobject obj)
{
  HMonitor my_monitor = MONITOR_getForObject(obj);

  if ( ! MONITOR_notifyOne(my_monitor) )
    {
      throw_Exception(env, "java/lang/IllegalMonitorStateException",
		      "not monitor owner!");
      return;
    }      
}

JNIEXPORT void JNICALL
Java_java_lang_Object_notifyAll(JNIEnv *env,
				jobject obj)
{
  HMonitor my_monitor = MONITOR_getForObject(obj);

  if ( ! MONITOR_notifyAll(my_monitor) )
    {
      throw_Exception(env, "java/lang/IllegalMonitorStateException",
		      "not monitor owner!");
      return;
    }      
}

/* XXX Should throw InterruptedException if interrupted */
JNIEXPORT void JNICALL
Java_java_lang_Object_wait(JNIEnv *env,
			   jobject obj,
			   jlong timeout)
{
  HMonitor my_monitor = MONITOR_getForObject(obj);
  if (0 > timeout)
    {
      throw_Exception(env, "java/lang/IllegalArgumentException",
		      "negative timeout");
      return;
    }      

  if ( ! MONITOR_timedWait(my_monitor, timeout) )
    {
      throw_Exception(env, "java/lang/IllegalMonitorStateException",
		      "not monitor owner!");
      return;
    }      
}
